<?php
namespace Teknisa\CRM\Controller;

use Zeedhi\Framework\DataSource\DataSet;
use Doctrine\ORM\EntityManager;
use Zeedhi\Framework\DTO\Request\Row;
use Zeedhi\Framework\DTO\Request\Filter;
use Zeedhi\Framework\DTO\Response;


class ControllerGeneric
{
    protected  $entityManager;

    public function __construct(EntityManager $entityManager, $environment)
    {
        $this->entityManager = $entityManager;
        $this->environment = $environment;
    }

    public function getParceiroNegocio(Filter $row, Response $response)
    {
    	$row = $row->getFilterCriteria()->getConditions();
        $param = array();
        foreach($row as $rw)
        {
            $param[$rw["columnName"]] = $rw["value"];
        }
        $sql = "SELECT NMPRINCIPALPARC AS NAME, NRPARCNEGOCIO AS VALUE FROM PARCNEGOCIO WHERE IDATIVO = 'S' AND NRORG = :NRORG AND EXISTS (SELECT NRPARCNEGOCIO FROM PARCERIA WHERE PARCERIA.NRPARCNEGOCIO = PARCNEGOCIO.NRPARCNEGOCIO AND PARCERIA.NRORG = PARCNEGOCIO.NRORG AND (PARCERIA.CDTIPOPARCEIRO = 'CLIENTE' OR (PARCERIA.CDTIPOPARCEIRO = 'INTERLOCUTOR' AND PARCNEGOCIO.NRPARCNEGOCIO NOT IN (SELECT NRPARCNEGRELAC FROM RELACIONAPARC))) AND PARCERIA.IDATIVO = 'S')";
        $param["NRORG"] = $this->environment->getNrOrgTrab();
    	$std = $this->entityManager->getConnection()->prepare($sql);
    	$std->execute($param);
    	$result = $std->fetchAll();
    	$response->addDataSet(new DataSet("/generic/Parceiro",$result));
	}

    public function getSistema(Filter $row, Response $response)
    {
        $row = $row->getFilterCriteria()->getConditions();
        $sql = "SELECT SI.CDSISTEMA AS VALUE, SI.NMSISTEMA AS NAME FROM SISTEMA SI, CRM_PERFISISTEMA PE, CRM_PERFIOPER PO WHERE SI.CDSISTEMA = PE.CDSISTEMA AND SI.NRORG = PE.NRORG AND PO.NRSEQPERFATIV = PE.NRSEQPERFATIV AND PO.NRORG = PE.NRORG AND PO.CDOPERADOR = :CDOPERADOR AND (SI.NRORG = 0 OR SI.NRORG = :NRORG) AND SI.IDATIVO = 'S' AND SI.IDTECNOSIST = 'WEB' AND SI.IDTIPLINPSIST = 'PROD' ORDER BY SI.NMSISTEMA";
        $std = $this->entityManager->getConnection()->prepare($sql);
        $param = array();
        foreach($row as $rw)
        {
            $param[$rw["columnName"]] = $rw["value"];
        }

        $param["NRORG"] = $this->environment->getNrOrgTrab();
        $std->execute($param);
        $result = $std->fetchAll();
        $response->addDataSet(new DataSet("/generic/Sistema",$result));
    }

    public function getSituacao(Filter $row, Response $response)
    {
        $row = $row->getFilterCriteria()->getConditions();
        $param = array();
        foreach($row as $rw)
        {
            $param[$rw["columnName"]] = $rw["value"];
        }

        $sql = "SELECT DSSITUACAO AS NAME, NRSEQSITUACAO AS VALUE FROM CRM_SITUACAO WHERE IDATIVO = 'S' AND (NRORG = 0 OR NRORG = :NRORG)";
        $std = $this->entityManager->getConnection()->prepare($sql);
        $param["NRORG"] = $this->environment->getNrOrgTrab();
        $std->execute($param);

        $result = $std->fetchAll();
        $response->addDataSet(new DataSet("/generic/Situacao",$result));
    }

    public function getClassificacaoCaso(Filter $row, Response $response)
    {
        $row = $row->getFilterCriteria()->getConditions();
        $param = array();
        foreach($row as $rw)
        {
            $param[$rw["columnName"]] = $rw["value"];
        }

        $sql = "SELECT DSCLASSIFICACAOCASO AS NAME, NRCLASSIFICACAOCASO AS VALUE FROM CRM_CLASSIFICACAOCASO WHERE IDATIVO = 'S' AND (NRORG = 0 OR NRORG = :NRORG)";
        $std = $this->entityManager->getConnection()->prepare($sql);
        $param["NRORG"] = $this->environment->getNrOrgTrab();
        $std->execute($param);

        $result = $std->fetchAll();
        $response->addDataSet(new DataSet("/generic/ClassificacaoCaso",$result));
    }

    public function getOperador(Filter $row, Response $response)
    {
        $row = $row->getFilterCriteria()->getConditions();
        $param = array();
        foreach($row as $rw)
        {
            $param[$rw["columnName"]] = $rw["value"];
        }

        $sql = "SELECT NMOPERADOR AS NAME, CDOPERADOR AS VALUE FROM OPERADOR WHERE IDATIVO = 'S' AND PLACEHOLDER";
        $sql = $this->trataSql($sql, $param);

        $std = $this->entityManager->getConnection()->prepare($sql);


        $std->execute($param);
        $result = $std->fetchAll();
        $response->addDataSet(new DataSet("/generic/Operador",$result));
    }

    public function getInterlocutor(Filter $row, Response $response)
    {
        $row = $row->getFilterCriteria()->getConditions();
        $param = array();
        foreach($row as $rw)
        {
            $param[$rw["columnName"]] = $rw["value"];
        }
        $sql = "SELECT NMPRINCIPALPARC AS NAME, NRPARCEIRO           AS VALUE FROM (SELECT RELACIONAPARC.NRPARCNEGRELAC AS NRPARCEIRO, PARCNEGOCIO.NMPRINCIPALPARC,     PARCNEGOCIO.NMSECUNDARIPARC, PARCNEGOCIO.NRORG, PARCNEGOCIO.IDATIVO, TO_CHAR(RELACIONAPARC.NRRELACIONAPARC) AS NRRELACIONAPARC, 'I' AS TIPO FROM PARCNEGOCIO, RELACIONAPARC WHERE PARCNEGOCIO.NRPARCNEGOCIO   = RELACIONAPARC.NRPARCNEGRELAC AND RELACIONAPARC.NRTIPORELACIONA = 25 AND RELACIONAPARC.NRPARCNEGOCIO   = NVL(:PARCEIRO,-1) UNION ALL SELECT PN.NRPARCNEGOCIO AS NRPARCEIRO, PN.NMPRINCIPALPARC, PN.NMSECUNDARIPARC, PN.NRORG, PN.IDATIVO,  ''  AS NRRELACIONAPARC, 'O' AS TIPO FROM PARCNEGOCIO PN INNER JOIN OPERADOR OP   ON PN.NRPARCNEGOCIO  = OP.NRPARCNEGOCIO WHERE OP.IDOPERATIVO = 'A' AND OP.CDFUNC       IS NOT NULL  ) PARCNEGOCIO ORDER BY TIPO, NMPRINCIPALPARC";

        $std = $this->entityManager->getConnection()->prepare($sql);


        $std->execute($param);
        $result = $std->fetchAll();
        $response->addDataSet(new DataSet("/generic/Interlocutor",$result));
    }

    public function getMotivo(Filter $row, Response $response)
    {
        $row = $row->getFilterCriteria()->getConditions();
        $param = array();

        foreach($row as $rw)
        {
            $param[$rw["columnName"]] = $rw["value"];
        }

        $sql = "SELECT MO.CDMOTICONT AS VALUE,  MO.NMMOTICONT      AS NAME,  MO.NRORG           AS NRORG FROM MOTICONT MO INNER JOIN CRM_PERFIMOTICONT PE ON MO.CDMOTICONT = PE.CDMOTICONT AND MO.NRORG     = PE.NRORG INNER JOIN CRM_PERFIOPER PO ON PO.NRSEQPERFATIV = PE.NRSEQPERFATIV AND PO.NRORG        = PE.NRORG AND PO.CDOPERADOR   = :CDOPERADOR WHERE MO.IDATIVO    = 'S' AND EXISTS (SELECT * FROM CRM_RELACIONAMOTIVOSISTEMA C WHERE MO.CDMOTICONT = C.CDMOTICONT AND C.CDSISTEMA     = :CDSISTEMA) UNION ALL SELECT MO.CDMOTICONT AS CDMOTICONT,  MO.NMMOTICONT AS NMMOTICONT, MO.NRORG AS NRORG FROM MOTICONT MO INNER JOIN CRM_PERFIMOTICONT PE ON MO.CDMOTICONT = PE.CDMOTICONT AND MO.NRORG     = PE.NRORG INNER JOIN CRM_PERFIOPER PO ON PO.NRSEQPERFATIV = PE.NRSEQPERFATIV AND PO.NRORG = PE.NRORG AND PO.CDOPERADOR   = :CDOPERADOR WHERE MO.IDATIVO    = 'S' AND NOT EXISTS (SELECT * FROM CRM_RELACIONAMOTIVOSISTEMA C WHERE C.CDSISTEMA = :CDSISTEMA)";

        $std = $this->entityManager->getConnection()->prepare($sql);

        $std->execute($param);
        $result = $std->fetchAll();
        $response->addDataSet(new DataSet("/generic/Motivo",$result));
    }

    public function getAllMotivos(Filter $row, Response $response)
    {
        $row = $row->getFilterCriteria()->getConditions();
        $param = array();

        foreach($row as $rw)
        {
            $param[$rw["columnName"]] = $rw["value"];
        }

        $sql = "SELECT MO.CDMOTICONT AS VALUE, MO.NMMOTICONT AS NAME, MO.NRORG AS NRORG FROM MOTICONT MO INNER JOIN CRM_PERFIMOTICONT PE ON MO.CDMOTICONT = PE.CDMOTICONT AND MO.NRORG = PE.NRORG INNER JOIN CRM_PERFIOPER PO ON PO.NRSEQPERFATIV = PE.NRSEQPERFATIV AND PO.NRORG = PE.NRORG AND PO.CDOPERADOR = :CDOPERADOR WHERE MO.IDATIVO = 'S'";

        $std = $this->entityManager->getConnection()->prepare($sql);

        $std->execute($param);
        $result = $std->fetchAll();
        $response->addDataSet(new DataSet("/generic/Motivo",$result));
    }

    public function getTipoContato(Filter $row, Response $response)
    {
        $row = $row->getFilterCriteria()->getConditions();
        $param = array();

        foreach($row as $rw)
        {
            $param[$rw["columnName"]] = $rw["value"];
        }

        $sql = "SELECT NMTIPOCONT AS NAME, CDTIPOCONT AS VALUE FROM TIPOCONT WHERE IDATIVO = 'S' AND (NRORG = 0 OR NRORG = :NRORG) ORDER BY NMTIPOCONT";

        $std = $this->entityManager->getConnection()->prepare($sql);
        $param["NRORG"] = $this->environment->getNrOrgTrab();

        $std->execute($param);
        $result = $std->fetchAll();
        $response->addDataSet(new DataSet("/generic/TipoContato",$result));
    }

    public function getMotivoContatoEvol(Filter $row, Response $response)
    {
        $row = $row->getFilterCriteria()->getConditions();
        
        $param = array();

        foreach($row as $rw)
        {
            $param[$rw["columnName"]] = $rw["value"];
        }

        $sql = "SELECT MO.CDMOTIEVOCON AS VALUE,MO.NMMOTIEVOCON AS NAME, MO.NRORG AS NRORG FROM MOTIEVOLCONT MO, CRM_PERFIMOTIEVO PE, CRM_PERFIOPER PO WHERE MO.CDMOTIEVOCON = PE.CDMOTIEVOCON AND MO.NRORG = PE.NRORG AND PO.NRSEQPERFATIV = PE.NRSEQPERFATIV AND PO.NRORG = PE.NRORG AND PO.CDOPERADOR = :CDOPERADOR AND MO.NRORG = :NRORG AND MO.IDATIVO = 'S'";

        $std = $this->entityManager->getConnection()->prepare($sql);
        $param["NRORG"] = $this->environment->getNrOrgTrab();

        $std->execute($param);
        $result = $std->fetchAll();
        $response->addDataSet(new DataSet("/generic/MotivoEvol",$result));
    }

    public function getInterlocutorEvol(Filter $row, Response $response)
    {
        $row = $row->getFilterCriteria()->getConditions();
        $param = array();
        foreach($row as $rw)
        {
            $param[$rw["columnName"]] = $rw["value"];
        }
        $sql = "SELECT REL.NRRELACIONAPARC AS VALUE, RP.NMPRINCIPALPARC AS NAME, REL.NRORG FROM RELACIONAPARC REL, PARCNEGOCIO RP WHERE REL.NRPARCNEGRELAC = RP.NRPARCNEGOCIO AND REL.NRPARCNEGOCIO    = :PARCEIRO ORDER BY RP.NMPRINCIPALPARC";

        $std = $this->entityManager->getConnection()->prepare($sql);


        $std->execute($param);
        $result = $std->fetchAll();
        $response->addDataSet(new DataSet("/generic/Interlocutor",$result));
    }

    public function getResultadoEvol(Filter $row, Response $response)
    {
        $row = $row->getFilterCriteria()->getConditions();
        $param = array();
        foreach($row as $rw)
        {
            $param[$rw["columnName"]] = $rw["value"];
        }
        $sql = "SELECT DISTINCT RE.NRSEQRESULEVOL AS VALUE, RE.DESCRESULEVOL AS NAME, RE.NRORG AS NRORG, RE.IDRESULTEVOLUCAO AS IDRESULTEVOLUCAO, RE.IDRESULTCASO AS IDRESULTCASO, RE.IDGERPROXATIV AS IDGERPROXATIV FROM CRM_RESULTEVOL RE, CRM_PERFIRESULT PR, CRM_PERFIOPER PO WHERE RE.NRSEQRESULEVOL = PR.NRSEQRESULEVOL AND RE.NRORG = PR.NRORG AND PO.NRSEQPERFATIV = PR.NRSEQPERFATIV AND PO.NRORG = PR.NRORG AND PO.CDOPERADOR = :CDOPERADOR AND RE.NRORG = :NRORG AND RE.IDATIVO = 'S' AND RE.IDRESULTEVOLUCAO = 'SIM'";

        $std = $this->entityManager->getConnection()->prepare($sql);
        $param["NRORG"] = $this->environment->getNrOrgTrab();

        $std->execute($param);
        $result = $std->fetchAll();
        $response->addDataSet(new DataSet("/generic/ResultadoEvol",$result));
    }

    public function getResultadoConclusao(Filter $row, Response $response)
    {
        $row = $row->getFilterCriteria()->getConditions();
        $param = array();
        foreach($row as $rw)
        {
            $param[$rw["columnName"]] = $rw["value"];
        }
        $sql = "SELECT DISTINCT RE.NRSEQRESULEVOL AS VALUE, RE.DESCRESULEVOL AS NAME, RE.NRORG AS NRORG, RE.IDRESULTEVOLUCAO AS IDRESULTEVOLUCAO, RE.IDRESULTCASO  AS IDRESULTCASO FROM CRM_RESULTEVOL RE, CRM_PERFIRESULT PR, CRM_PERFIOPER PO WHERE RE.NRSEQRESULEVOL = PR.NRSEQRESULEVOL AND RE.NRORG = PR.NRORG AND PO.NRSEQPERFATIV = PR.NRSEQPERFATIV AND PO.NRORG = PR.NRORG AND PO.CDOPERADOR = :CDOPERADOR AND RE.NRORG = :NRORG AND RE.IDATIVO = 'S' AND RE.IDRESULTCASO = 'SIM'";

        $std = $this->entityManager->getConnection()->prepare($sql);
        $param["NRORG"] = $this->environment->getNrOrgTrab();

        $std->execute($param);
        $result = $std->fetchAll();
        $response->addDataSet(new DataSet("/generic/ResultadoConclusao",$result));
    }

    public function trataSql($sql, $params)
    {
        $string = "1 = 1 AND ";
        foreach($params as $key => $param)
        {
           $string.= $key." = :".$key." AND ";
        }

        $string = substr($string, 0 , -4);
        $sql = str_replace("PLACEHOLDER", $string, $sql);

        return $sql;
    }

    public function retornaOperador(Row $row, Response $response)
    {
        $row = $row->getRow();
        $select = "SELECT NMOPERADOR FROM OPERADOR WHERE CDOPERADOR = :CDOPERADOR AND NRORG = :NRORG AND IDATIVO = 'S'";
        $std = $this->entityManager->getConnection()->prepare($select);
        $params = array("CDOPERADOR" => $row["valor"][0], "NRORG" => $row["valor"][1]);
        $params["NRORG"] = $this->environment->getNrOrgTrab();
        $std->execute($params);
        $return = $std->fetchAll();
        $response->addDataSet(new DataSet("operador",$return));
    }


}
?>