<?php
namespace Teknisa\CRM\Controller;

use Teknisa\CRM\Controller\TeknisaCrud;
use Zeedhi\Framework\Controller\Crud;
use Zeedhi\Framework\DataSource\Manager;
use Zeedhi\Framework\DataSource\ParameterBag;
use Zeedhi\Framework\DataSource\FilterCriteria;
use Zeedhi\Framework\DTO;
use Zeedhi\Framework\DataSource\DataSet;
use Zeedhi\Framework\DTO\Request;
use HCM\Util\HCMParameters;
use HCM\Util\Messages;

class EvolucaoController extends TeknisaCrud {
    //protected $dataSourceName = 'evolucao';
    protected $primaryKey = 'NRSEQEVOLCONT';
    //protected $CDCONTADOR = 'EVOLCONT';

    protected $agendamento_dataSourceName = 'filacontato';
    protected $dataSourceName = 'evolucaoConsulta';
    protected $agendamento_primaryKey = 'NRSEQFILACONTA';
    protected $agendamento_CDCONTADOR = 'FILACONTATO';
    protected $environment;
    protected $evolucaoService;
    protected $parameterBagColumns = array('P_NRORG', 'P_NRORG_PADRAO',
        "IDTPOPERADOR", "NRREVENDA", "P_DTINICIO", "P_DTFIM");

    public function __construct($crudService, $parameterBag, $environment, $evolucaoService, $zhFileUploader, $anexoController, $enviarMensagemController, $util){
        parent::__construct($crudService, $parameterBag);
        $this->environment              = $environment;
        $this->evolucaoService          = $evolucaoService;
        $this->zhFileUploader           = $zhFileUploader;
        $this->anexoController          = $anexoController;
        $this->enviarMensagemController = $enviarMensagemController;
        $this->util                     = $util;
    }
    
    public function find(DTO\Request\Filter $request, DTO\Response $response) {
        $request = $this->util->modifyDateFilter($request, 'DTHREVOLCONT');
        $filterData = $request->getFilterCriteria();
        $filterData->setPageSize(99999);
        
        parent::find($request, $response);
    }
    
    public function save(DTO\Request $request, DTO\Response $response) {
        foreach($request->getDataSet()->getRows() as $row) {
            $row["CDFILIAL"] = $this->environment->getCdFilial();
            if(!array_key_exists('NRRELACIONAPARC', $row)) $row['NRRELACIONAPARC'] = null;
            if(!array_key_exists('DSTELEFONETAREFA', $row)) $row['DSTELEFONETAREFA'] = null;
            if(!array_key_exists('DSEMAILTAREFA', $row)) $row['DSEMAILTAREFA'] = null;
            if(!array_key_exists('NRSEQCLASSIFICATAREFA', $row)) $row['NRSEQCLASSIFICATAREFA'] = null;
            if(!array_key_exists('DSDETEFILACONTA', $row)) $row['DSDETEFILACONTA'] = null;

            if($row['__is_new']){
                date_default_timezone_set('America/Sao_Paulo');
                $row['DTHREVOLCONT'] = date('d/m/y');

                $row['__new_code_pk'] = "EVOLCONT";
                $row['__table_pk'] = "NRSEQEVOLCONT";
                $row['__cast_number'] = true;
                $row['__new_code_length'] = 20;
            }
        }

    	parent::save($request, $response);

        if($request->getDataSet()->getRows()[0]['IDAGEEVOCONT'] == 'S') {
            foreach($request->getDataSet()->getRows() as $row) {
                if(!array_key_exists('CDOPERREAL', $row)) $row['CDOPERREAL'] = $row['CDOPEREVOL'];
            }

            $previous_dataSourceName = $this->dataSourceName;
            $previous_primaryKey = $this->primaryKey;
            $previous_CDCONTADOR = $this->CDCONTADOR;

            $this->dataSourceName = $this->agendamento_dataSourceName;
            $this->primaryKey = $this->agendamento_primaryKey;
            $this->CDCONTADOR = $this->agendamento_CDCONTADOR;

            parent::save($request, $response);

            $this->dataSourceName = $previous_dataSourceName;
            $this->primaryKey = $previous_primaryKey;
            $this->CDCONTADOR = $previous_CDCONTADOR;
        }
    
        
        
    }
    
    protected function processConditionValue($condition) {
        return $condition['value'];
    }
    protected function populateParameterBag(FilterCriteria $filterCriteria) {

        $conditions = array();
        foreach ($filterCriteria->getConditions() as $condition) {
            $columnName = $condition['columnName'];
            if (in_array($columnName, $this->parameterBagColumns)) {
                $this->parameterBag->set($columnName, $this->processConditionValue($condition));
            } else {
                $conditions[] = $condition;
            }
        }
        $newFilterCriteria = new FilterCriteria(
            $filterCriteria->getDataSourceName(),
            $conditions,
            $filterCriteria->getPage(),
            $filterCriteria->getPageSize()
        );
        return $newFilterCriteria;
    }
    
    public function getNrDiasLanceEvol($request, DTO\Response $response){
        $row = $request->getRow();
        
        $paramsSelectNrDiasLanceEvol = array(
            'NRORG'             => $row['NRORG']
        );
        
        $nrDiasLanceEvol = $this->evolucaoService->callForGetNrDiasLanceEvol($paramsSelectNrDiasLanceEvol);
        $dataSetResponse = new DataSet("DADOS",array("NRDIASLANCEVOL" => $nrDiasLanceEvol));
        
        $response->addDataSet($dataSetResponse);
    }
    
    public function notificaInterlocutorEvolucao($request, DTO\Response $response){
        
        $from = $this->environment->getCdOperador();
        $row = $request->getRow();
        $numberOfFiles = isset($row['DSCAMINHO']) ? count($row['DSCAMINHO']) : 0;
        
        $mensagem = array();
        $mensagem['DSMENSAGEM']       = $row['DSMENSAGEM'];
        $mensagem['DSTITULOMENSAGEM'] = empty($row['DSTITULOMENSAGEM']) ? 'CRM - Notificação de Evolução' : $row['DSTITULOMENSAGEM'];
        $mensagem['INTERLOCUTOR']     = ucwords(strtolower($row['NMINTERLOCUTOR']));   
        $mensagem['DATAEVOLUCAO']     = $row['DATAEVOLUCAO'];
        
        $attach = $this->evolucaoService->getAttachment($row['NRORG'], $row['NRSEQEVOLCONT'],$numberOfFiles);
        $this->enviarMensagemController->enviarEmail($mensagem, $row['NRORG'], null, $from, $row['DSEMAILINTERLOC'], $row['NMINTERLOCUTOR'], null, $attach);
       
    }
    
    public function saveAttachEvol($request, DTO\Response $response){
        $index=0;
        $save = false;
        foreach ($request->getDataSet()->getRows() as &$attachment) {
            if(!isset($attachment['DEFAULTATTACH']) && array_key_exists($index, $attachment['DSCAMINHO'])){
                $file = array(
                    'FILE' => array(
                        array(
                            "name" => $attachment['DSCAMINHO'][$index]['name'],
                            "type" => explode('.', $attachment['DSCAMINHO'][$index]['name'])[1],
                            "b64File" => $attachment['DSCAMINHO'][$index]['b64File'],
                            "size" => $attachment['DSCAMINHO'][$index]['size']
                        )
                    )    
                );
                $this->zhFileUploader->uploadFilesInRow($file, 'FILE', 'crmanexocaso');
                $attachment['DSCAMINHO'][$index]['path'] = $file['FILE'][0]['path'];
                $save = true;
            }else{
                $attachment['VRTAMANHO'] = null;
            }
            $attachment['__is_new'] = true;
            $index++;
        }
        if($save) $this->anexoController->save($request, $response);
    }
    
}