/* global ElementToMetadataService, Container, Field, Action, Menu, Widget inject expect */
describe("ElementToMetadataService", function() {
	var elementToMetadata;
	var templateManager;
	var container;
	var action;
	var field;
	var menu;
	var widget;
	var groupMenu;
	var project;
	var DELETE = "DELETE";
	var NEW = "NEW";
	var DELETEWIDGET = "DELETEWIDGET";

	beforeEach(function (){
		module('ZeedhiApp');
		inject(function(_templateManager_){
			templateManager=_templateManager_;
			container = {
				"name": "default",
				"label": "default",
				"isVisible": true,
				"showFooter": true,
				"showHeader": true,
				"showMenu": true,
				"footer": "component/footer.html",
				"template": "container/window.html",
				"widgets": [
					{
						"name": "default",
						"label": "default",
						"template": "widget/form-login.html"
					}
				]
			};
			menu = {
				"showInDevTools":true,
				"name":"value",
				"windowName":"menu",
				"parent":{}
			};
			groupMenu = [
				{
					"label":"teste",
					"isVisible":true,
					"menus":[]
				}
			];
			project = {
				"hasChild":true,
				"id":"20169762992902656303854",
				"label":"teste",
				"menus":[],
				"name":"teste",
				"parentMenu":{},
				"showInDevTools":true
			};
			templateManager.container = container;
			templateManager.project = project;
			templateManager.project.groupMenu = groupMenu;
			elementToMetadata = new ElementToMetadataService(templateManager);
			field = {"name":"default","id":"default","isVisible":true};
			action = {"name":"default","id":"default","isVisible":true};
			widget = {"name":"default","id":"default","isVisible":true,"fields":[],"actions":[]};
		});
	});

	it("HasChanged() of different objects and without /fileName should return true", function (){
		var oldObj={"unknownProperty":"value"};
		var newObj={};
		expect(elementToMetadata.hasChanged(oldObj,newObj)).toBe(true);
	});

	it("HasChanged() of different objects and with /fileName should return true", function(){
		var oldObj={"unknownProperty":"value","fileName":"default"};
		var newObj={};
		expect(elementToMetadata.hasChanged(oldObj,newObj)).toBe(true);
	});

	it("HasChanged() of a Object and undefined should return true",function (){
		var oldObj = {"unknownProperty":"value"};
		var newObj = undefined;
		expect(elementToMetadata.hasChanged(oldObj,newObj)).toBe(true);
	});

	it("HasChanged() of equal objects and without /fileName should return false",function (){
		var oldObj = {"unknownProperty":"value"};
		var newObj = {"unknownProperty":"value"};
		expect(elementToMetadata.hasChanged(oldObj,newObj)).toBe(false);
	});

	it("HasChanged() of equal objects and with /fileName should return false", function(){
		var oldObj={"unknownProperty":"value","fileName":"default"};
		var newObj={"unknownProperty":"value"};
		expect(elementToMetadata.hasChanged(oldObj,newObj)).toBe(false);
	});

	it("GetPathToMetadata() should return as path an empty array when the parameter's Object isn't a property from propertiesMap", function(){
		expect(elementToMetadata.getPathToMetadata({"notification":"clock"})).toEqual([]);
	});

	it("GetPathToMetadata() should return as path an empty array when the parameter's Object is the parent container", function(){
		expect(elementToMetadata.getPathToMetadata(container)).toEqual([]);
	});

	it("GetPathToMetadata() of a action should return the path of the action", function(){
		var actualAction = new Action();
		var anotherAction = new Action();
		actualAction.widget = new Widget();
		anotherAction.widget = actualAction.widget;
		actualAction.widget.container = new Container();
		anotherAction.widget.container = actualAction.widget.container;
		actualAction.widget.container.widgets = [actualAction.widget];
		anotherAction.widget.container.widgets = actualAction.widget.container.widgets;
		actualAction.widget.actions = [anotherAction,actualAction];
		var thisAction = {"action":actualAction};
		expect(elementToMetadata.getPathToMetadata(thisAction)).toEqual(["widgets",0,"actions",1]);
	});

	it("GetPathToMetadata() of a field should return the path of the field", function(){
		var actualField = new Field();
		actualField.widget = new Widget();
		actualField.widget.container = new Container();
		actualField.widget.container.widgets = [actualField.widget];
		actualField.widget.fields = [actualField];
		var thisField = {"field":actualField};
		expect(elementToMetadata.getPathToMetadata(thisField)).toEqual(["widgets",0,"fields",0]);
	});

	it("GetPathToMetadata() of a child widget should return the path of the child widget", function(){
		var actualWidget = new Widget();
		actualWidget.container = new Container();
		actualWidget.container.widgets = [actualWidget];
		var thisWidget = {"widget":actualWidget};
		expect(elementToMetadata.getPathToMetadata(thisWidget)).toEqual(["widgets",0]);
	});

	it("GetPathToMetadata() of a child widget should return the path of the child widget", function(){
		var actualWidget = new Widget();
		actualWidget.container = new Container();
		actualWidget.container.widgets = [actualWidget];
		actualWidget.parent = actualWidget.container;
		var thisWidget = {"widget":actualWidget};
		expect(elementToMetadata.getPathToMetadata(thisWidget)).toEqual(["widgets",0]);
	});

	it("GetPathToMetadata() of a groupMenu should return the path of the groupMenu", function(){
		expect(elementToMetadata.getPathToMetadata(groupMenu)).toEqual(["groupMenu"]);
	});

	it("GetPathToMetadata() of a child menu should return the path of the child menu", function(){
		var actualMenu = new Menu();
		actualMenu.parentMenu = new Object();
		actualMenu.parentMenu.groupMenu = [actualMenu];
		actualMenu.windowName = "menu";
		var thisMenu = actualMenu;
		expect(elementToMetadata.getPathToMetadata(thisMenu)).toEqual(["groupMenu",0]);
	});

	it("GetPathToMetadata() of a child of a child menu should return the path of it", function(){
		var outsideMenu = new Menu();
		var insideMenu = new Menu();
		outsideMenu.windowName = "value";
		outsideMenu.menus = [insideMenu];
		outsideMenu.parentMenu = new Object();
		outsideMenu.parentMenu.groupMenu = [outsideMenu];
		insideMenu.windowName = "value";
		insideMenu.parent = outsideMenu;
		var thisMenu = insideMenu;
		expect(elementToMetadata.getPathToMetadata(thisMenu)).toEqual(["groupMenu",0,"menus",0]);
	});

	it("GetPathToMetadata() of a child of a child menu should return the path of it passing by the element.parent.menus at getMenuType", function(){
		var outsideMenu = new Menu();
		var insideMenu = new Menu();
		outsideMenu.windowName = "value";
		outsideMenu.menus = [insideMenu];
		outsideMenu.parentMenu = new Object();
		outsideMenu.parentMenu.groupMenu = [outsideMenu];
		insideMenu.windowName = "value";
		insideMenu.parent = outsideMenu;
		var thisMenu = insideMenu;
		expect(elementToMetadata.getPathToMetadata(thisMenu)).toEqual(["groupMenu",0,"menus",0]);
	});

	it("GetPathToMetadata() should return a empty array as path when there's no parameter's paremeter",function(){
		expect(elementToMetadata.getPathToMetadata()).toEqual([]);
	});

	it("GetPathToMetadata() should return a empty array as path when the parameter is undefined",function(){
		expect(elementToMetadata.getPathToMetadata(undefined)).toEqual([]);
	});

	it("GetPropertyFromScope() should return the property's name == 'container' from the scope's instance of devToolsWindow", function() {
		expect(elementToMetadata.getPropertyFromScope(new Container())).toBe("container");
	});

	it("GetPropertyFromScope() should return the property's name == 'menu' from the scope's instance of devToolsWindow", function() {
		expect(elementToMetadata.getPropertyFromScope(new Menu())).toBe("menu");
	});

	it("GetPropertyFromScope() should return the property's name == 'field' from the scope's instance of devToolsWindow", function() {
		expect(elementToMetadata.getPropertyFromScope(new Field())).toBe("field");
	});

	it("GetPropertyFromScope() should return the property's name == 'action' from the scope's instance of devToolsWindow", function() {
		expect(elementToMetadata.getPropertyFromScope(new Action())).toBe("action");
	});

	it("GetPropertyFromScope() should return the property's name == 'widget' from the scope's instance of devToolsWindow", function() {
		expect(elementToMetadata.getPropertyFromScope(new Widget())).toBe("widget");
	});

	it("GetPropertyFromScope() should return the property's name == 'column' from the scope's instance of devToolsWindow",function(){
		var devToolsWindow = window.opener || window.parent;
		devToolsWindow.Column =function(ZhUtil, templateManager, DecodeValueFieldService){};
		var column = new devToolsWindow.Column;
		expect(elementToMetadata.getPropertyFromScope(column)).toBe("column");
	});

	it("GetPropertyFromScope() should return false since groupMenu isn't an instance of devToolsWindow but an array", function() {
		expect(elementToMetadata.getPropertyFromScope(groupMenu)).toBe(false);
	});

	it("GetPropertyFromScope() should return false when the property doesn't exist in propertiesMap", function (){
		var objectOwnerOfTheProperty = {"unknownProperty":"value"};
		expect(elementToMetadata.getPropertyFromScope(objectOwnerOfTheProperty)).toBe(false);
	});

	it("GetPropertyFromScope() should return false when the Object made to be a menu isn't a instance of the class Menu()", function() {
		expect(elementToMetadata.getPropertyFromScope(menu)).toBe(false);
	});

	it("GetPropertyFromScope() should return false when the Object made to be a field isn't a instance of the class Field()", function() {
		expect(elementToMetadata.getPropertyFromScope(field)).toBe(false);
	});

	it("GetPropertyFromScope() should return false when the Object made to be a action isn't a instance of the class Action()", function() {
		expect(elementToMetadata.getPropertyFromScope(action)).toBe(false);
	});

	it("GetPropertyFromScope() should return false when the Object made to be a widget isn't a instance of the class Widget()", function() {
		expect(elementToMetadata.getPropertyFromScope(widget)).toBe(false);
	});

	it("GetPropertyFromScope() should return false when there's no paremeter", function() {
		expect(elementToMetadata.getPropertyFromScope()).toBe(false);
	});

	it("GetPropertyFromScope() should return false when the paremeter is undefined", function() {
		expect(elementToMetadata.getPropertyFromScope(undefined)).toBe(false);
	});

	it("GetPropertyFromScope() should return false when devToolsWindow isn't open", function (){
		var testObject;
		window.parent = false;
		inject(function(_templateManager_){
			templateManager=_templateManager_;
			templateManager.inDeveloperMode = false;
			testObject = new ElementToMetadataService(templateManager);
		});
		expect(testObject.getPropertyFromScope(new Field())).toBe(false);
	});

	it("ChangeJson() without changes should return an Object with the property completeJson equals the parameter's completeJson", function(){
		var json = {
			"label": "value"
		};
		var path = ["widgets",0];
		var completeJson = {
			"widgets": [
				{
					"label": "value"
				}
			]
		};
		var responseObject = elementToMetadata.changeJson(JSON.stringify(json), path, completeJson, undefined);
		expect(responseObject.completeJson).toBe(completeJson);
	});

	it("ChangeJson() with changes should return an Object with the property completeJson different from the parameter's completeJson", function(){
		var json = {
			"label": "valuE"
		};
		var path = ["widgets",0];
		var completeJson = {
			"widgets": [
				{
					"label": "value"
				}
			]
		};
		var responseObject = elementToMetadata.changeJson(JSON.stringify(json), path, Util.clone(completeJson), undefined);
		expect(responseObject.completeJson).not.toBe(completeJson);
	});

	it("ChangeJson() with DELETE option as parameter should return an Object with the property completeJson without the excluded part", function(){
		var path = ["widgets",0,"fields",0];
		var completeJson = {
			"widgets":
			[
				{
					"fields":
					[
						{"name":"value"}
					]
				}
			]
		};
		var json = {"name":"value"};
		var responseObject = elementToMetadata.changeJson(JSON.stringify(json), path, completeJson, DELETE);
		expect(JSON.stringify(responseObject.completeJson)).toBe("{\"widgets\":[{\"fields\":[]}]}");
	});

	it("ChangeJson() with DELETE option and undefined path as paremeters should return an Object with completeJson equals undefined", function(){
		var completeJson = {
			"widgets":
			[
				{
					"fields":
					[
						{"name":"value"}
					]
				}
			]
		};
		var responseObject = elementToMetadata.changeJson(undefined, undefined, completeJson, DELETE);
		expect(JSON.stringify(responseObject.completeJson)).toBe(undefined);
	});

	it("ChangeJson() with DELETE option and COMPLETEJSON equal JSON as parameters should return an Object with the property somethingChange equals true", function(){
		var path = ["widgets",0,"fields",0];
		var completeJson = {
			"widgets":
			[
				{
					"fields":
					[
						{"name": "value"}
					]
				}
			]
		};
		var responseObject = elementToMetadata.changeJson(JSON.stringify(completeJson), path, completeJson, DELETE);
		expect(responseObject.somethingChange).toBe(true);
	});


	it("ChangeJson() with DELETE option and undefined path as paremeters should return an Object with somethingChange equals true", function(){
		var completeJson = {
			"widgets":
			[
				{
					"fields":
					[
						{"name": "value"}
					]
				}
			]
		};
		var responseObject = elementToMetadata.changeJson(undefined, undefined, completeJson, DELETE);
		expect(responseObject.somethingChange).toBe(true);
	});

	it("ChangeJson() with DELETEWIDGET option as parameter should return an Object with the property completeJson without the excluded part", function(){
		var path = ["widgets",0,"widgets",0];
		var completeJson = {
			"widgets":
			[
				{
					"widgets":
					[
						{"name":"value"}
					]
				}
			]
		};
		var json = {"name":"value"};
		var responseObject = elementToMetadata.changeJson(JSON.stringify(json), path, completeJson, DELETEWIDGET);
		expect(JSON.stringify(responseObject.completeJson)).toBe("{\"widgets\":[{\"widgets\":[]}]}");
	});

	it("ChangeJson() with DELETEWIDGET option and undefined path as paremeters should return an Object with the property completeJson equals undefined", function(){
		var completeJson = {
			"widgets":
			[
				{
					"widgets":
					[
						{"name":"value"}
					]
				}
			]
		};
		var responseObject = elementToMetadata.changeJson(undefined, undefined, completeJson, DELETEWIDGET);
		expect(JSON.stringify(responseObject.completeJson)).toBe(undefined);
	});

	it("ChangeJson() with DELETEWIDGET option as parameter should return an Object with the property somethingChange equals true", function(){
		var path = ["widgets",0];
		var completeJson = {
			"widgets":
			[
				{
					"fields":
					[
						{"name":"value"}
					]
				}
			]
		};
		var json = {"name":"value"};
		var responseObject = elementToMetadata.changeJson(JSON.stringify(json), path, completeJson, DELETEWIDGET);
		expect(responseObject.somethingChange).toBe(true);
	});

	it("ChangeJson() with DELETEWIDGET option and undefined path as paremeters should return an Object with the property somethingChange equals true", function(){
		var completeJson = {
			"widgets":
			[
				{
					"fields":
					[
						{"name":"value"}
					]
				}
			]
		};
		var responseObject = elementToMetadata.changeJson(undefined,undefined, completeJson, DELETEWIDGET);
		expect(responseObject.somethingChange).toBe(true);
	});

	it("ChangeJson() with NEW option as parameter should return an Object with the property completeJson with the new JSON.Object", function(){
		var path = ["widgets",0];
		var json = {"name":"a"};
		var completeJson = {"widgets":[{}]};
		var responseObject = elementToMetadata.changeJson(JSON.stringify(json), path, completeJson, NEW);
		expect(JSON.stringify(responseObject.completeJson)).toBe("{\"widgets\":[{},{\"name\":\"a\"}]}");
	});

	it("ChangeJson() with NEW option and an undefined path as parameters should return an Object with the property completeJson with the new JSON", function(){
		var completeJson = {"widgets":[{}]};
		var json = {"name": "a"};
		var responseObject = elementToMetadata.changeJson(JSON.stringify(json), undefined, completeJson, NEW);
		expect(JSON.stringify(responseObject.completeJson)).toBe(JSON.stringify(json));
	});

	it("ChangeJson() with NEW option should return an Object with the property somethingChange equals true", function(){
		var path = ["widgets",0];
		var json = {"name":"a"};
		var completeJson = {"widgets":[{}]};
		var responseObject = elementToMetadata.changeJson(JSON.stringify(json), path, completeJson, NEW);
		expect(responseObject.somethingChange).toBe(true);
	});

	it("ChangeJson() with NEW option and undefined path as parameters should return an Object with the property somethingChange equals true", function(){
		var completeJson = {"widgets":[{}]};
		var responseObject = elementToMetadata.changeJson(undefined, undefined, completeJson, NEW);
		expect(responseObject.somethingChange).toBe(true);
	});

	it("ChangeJson() with changes and a undefined path should return an Object with the property somethingChange equals true",function() {
		var completeJson = {"widgets":[{}]};
		var json = {"name":"a"};
		var responseObject = elementToMetadata.changeJson(JSON.stringify(json), undefined, completeJson, undefined);
		expect(responseObject.somethingChange).toBe(true);
	});

	it("ChangeJson() with changes should return an Object with the property somethingChange equals true", function(){
		var json = {
			"label": "valuE"
		};
		var path = ["widgets",0];
		var completeJson = {
			"widgets": [
				{
					"label": "value"
				}
			]
		};
		var responseObject = elementToMetadata.changeJson(JSON.stringify(json), path, completeJson, undefined);
		expect(responseObject.somethingChange).toBe(true);
	});

	it("ChangeJson() without changes should return an Object with the property somethingChange equals false", function(){
		var json = {
			"label": "value"
		};
		var path = ["widgets",0];
		var completeJson = {
			"widgets": [
				{
					"label": "value"
				}
			]
		};
		var responseObject = elementToMetadata.changeJson(JSON.stringify(json), path, completeJson, undefined);
		expect(responseObject.somethingChange).toBe(false);
	});

	it("ChangeJson() without changes and a undefined path should return an Object with the property somethingChange equals false",function() {
		var completeJson = {"widgets":[{}]};
		var responseObject = elementToMetadata.changeJson(JSON.stringify(completeJson), undefined, completeJson, undefined);
		expect(responseObject.somethingChange).toBe(false);
	});

	it("ChangeJson() with all undefined entries should return an Object with the property somethingChange equals false",function() {
		var responseObject = elementToMetadata.changeJson(undefined, undefined, undefined, undefined);
		expect(responseObject.somethingChange).toBe(false);
	});

	it("ChangeJson() with all undefined entries should return an Object with the property completeJson equals undefined",function() {
		var responseObject = elementToMetadata.changeJson(undefined, undefined, undefined, undefined);
		expect(JSON.stringify(responseObject.completeJson)).toBe(undefined);
	});

	it("ChangeJson() without changes and a undefined path should return an Object with the property completeJson with the same JSON.Object as before",function() {
		var completeJson = {"widgets":[{}]};
		var responseObject = elementToMetadata.changeJson(JSON.stringify(completeJson), undefined, completeJson, undefined);
		expect(JSON.stringify(responseObject.completeJson)).toBe(JSON.stringify(completeJson));
	});

	it("ChangeJson() with changes and a undefined path should return an Object with the property completeJson having added the parameter's json parameter",function() {
		var completeJson = {"widgets":[{}]};
		var json = {"name":"a"};
		var responseObject = elementToMetadata.changeJson(JSON.stringify(json), undefined, completeJson, undefined);
		expect(JSON.stringify(responseObject.completeJson.name)).toBe(JSON.stringify(json.name));
	});

	it("GetJsonElement() should return an Object with the property jsonElement when the paremeter's JSON.Object exists", function(){
		var path = ["widgets",0];
		var objectContainer = new Container();
		objectContainer.__metadata = container;
		var responseObject = elementToMetadata.getJsonElement(path,objectContainer);
		expect(JSON.stringify(responseObject.jsonElement)).toBe(JSON.stringify(objectContainer.__metadata.widgets[0]));
	});

	it("GetJsonElement() should return an Object with the property's path when the paremeter's JSON.Object exists", function(){
		var path = ["widgets",0];
		var objectContainer = new Container();
		objectContainer.__metadata = container;
		var responseObject = elementToMetadata.getJsonElement(path,objectContainer);
		expect(responseObject.path).toEqual(["widgets",0]);
	});

	it("GetJsonElement() of a non parent property should enter in the path.every() should return an Object with the property path", function(){
		var path = ["widgets",0];
		var objectContainer = new Container();
		objectContainer.__metadata = container;
		var responseObject = elementToMetadata.getJsonElement(path,objectContainer);
		expect(responseObject.path).toEqual(["widgets",0]);
	});

	it("GetJsonElement() of a non parent property with no _metadata should enter in the path.every()'s if and should return false", function(){
		var path = ["widgets",0];
		var objectContainer = new Container();
		objectContainer.__metadata = undefined;
		expect(elementToMetadata.getJsonElement(path,objectContainer)).toBe(false);
	});

	it("GetJsonElement() should return false when the JsonElement doesn't exists", function(){
		var path = ["widgets",-1];
		var objectContainer = new Container();
		objectContainer.__metadata = container;
		expect(elementToMetadata.getJsonElement(path,objectContainer)).toBe(false);
	});

	it("GetElementToMetadata() of the parent container Object should return an Object with the property path equals an empty array", function(){
		var objectContainer = new Container();
		objectContainer.__metadata = container;
		var responseObject = elementToMetadata.getElementToMetadata(container,objectContainer);
		expect(responseObject.path).toEqual([]);
	});

	it("GetElementToMetadata() of the parent container Object should return an Object with the property JsonElement", function(){
		var objectContainer = new Container();
		objectContainer.__metadata = container;
		var responseObject = elementToMetadata.getElementToMetadata(container,objectContainer);
		expect(JSON.stringify(responseObject.jsonElement)).toBe(JSON.stringify(objectContainer.__metadata));
	});

});