/*global Mousetrap ZeedhiChart ace JSHINT jsonlint Util*/
ZeedhiChart.directive("zhCodeEditor", function(ScreenService, eventAggregator, DeveloperToolsForm) {
	return {
		restrict: "A",
		link: function($scope, $element, $attr, $ctrl) {
			function getKeepSession(newItem, oldItem){
				var keepSession = false;
				if(newItem && oldItem && newItem.path && oldItem.path){
					keepSession = newItem.path == oldItem.path && newItem.mode == oldItem.mode;
				}
				return keepSession;
			}

			function clearSession(text) {
				var newSession = ace.createEditSession(text || "", new $scope.currentModeClass());
				$scope.currentEditor.setSession(newSession);
				$scope.currentEditor.getSession().setTabSize(2);
			}

			function getErrorLog(errors){
				var log = "";
				errors.forEach(function(error, index){
					if(error){
						log += error.reason + "In line: " + error.line;
						log += "\n";
					} else if(errors[index-1] && errors[index-1].code == "E043"){
						log = errors[index-1].reason + "\n" + log;
					}
				});
				return log;
			}

			function getItemAttributes(item){
				var attributes = ['text', 'path', 'mode'];
				var objectItem = {};
				item.forEach(function(value, index){
					var indexName = attributes[index];
					if(indexName == 'path'){
						value = item[index] ? item[index].join('/') : "";
					}
					objectItem[indexName] = value;
				});
				return objectItem;
			}

			function isScriptWithValidation(path){
				return !(~path.indexOf('.css') || ~path.indexOf('.html'));
			}

			function isValidJson(text){
				jsonlint.parse(text);
			}

			function isValidJs(text){
				var validation = isScriptWithValidation($scope.getActiveItem()._indexPath);
				if(validation){
					JSHINT(text, { eqnull:true}, {jquery:true});
					if(JSHINT.errors.length > 0){
						throw new Error(getErrorLog(JSHINT.errors));
					}
				}
			}

			function isValid(text, validFunction){
				var annotations = $scope.currentEditor ? $scope.currentEditor.getSession().getAnnotations() : [];
				var aceLintErrors = annotations.filter(function(annotation){ return annotation.type === "error"; });
				var isValid = !aceLintErrors.length;
				if(isValid && text){
					try {
						validFunction(text);
						isValid = true;
					} catch (e){
						ScreenService.errorNotification(e.message);
						isValid = false;
					}
				}
				return isValid;
			}

			function setAnnotations(annotations) {
				$scope.currentEditor.getSession().setAnnotations(annotations);
			}

			function getActiveComponentState(){
				var state = {};
				if($scope.inFormMode()){
					state = DeveloperToolsForm.getActiveFormState($scope.activeItem, $scope.formWidget);
				} else {
					var text = $scope.currentEditor ? $scope.currentEditor.getValue() : "";
					var validFunction = $scope.inFileMode() ? isValidJs : isValidJson;
					state = {
						valid: isValid(text, validFunction),
						value: text
					};
				}
				return state;
			}

			function activeEditorSession(text, cursorPosition, items){
				if($scope.currentEditor){
					cursorPosition = cursorPosition || {row: 0, column: 0};
					if(!getKeepSession(items.newItem, items.oldItem)){
						text = $scope.getTextAndOpenLastPath(text, items);
						clearSession(text);
					} else if($scope.currentEditor.getValue() != text) {
						$scope.currentEditor.setValue(text);
					}
					$scope.currentEditor.gotoLine(cursorPosition.row + 1, cursorPosition.column);
					$scope.currentEditor.focus();
					$scope.textIsReady = true;
				}
			}

			$scope.setAnnotations = setAnnotations;
			$scope.getActiveComponentState = getActiveComponentState;
			$scope.activeEditorSession = activeEditorSession;

			$scope.clearEditor = function() {
				if($scope.currentEditor){
					clearSession();
					$scope.currentEditor.setValue("");
					$scope.currentEditor.gotoLine(0,0);
				}
			};

			$scope.$watch(function(){
				return $element.children()[0];
			},function(){
				$scope.$evalAsync(function(){
					if($element.children()[0]){
						$scope.initEditor();
					}
				});
			});

			function watchActiveItem(){
				return $scope.activeItem ? [$scope.activeItem.text, $scope.activeItem.path, $scope.currentMode] : [$scope.activeItem];
			}

			function activeItemHandler(newItem, oldItem){
				var activeItem = $scope.activeItem;
				if(activeItem){
					var text = activeItem.text;
					var annotations = activeItem.errorMessage ? [activeItem.errorMessage] : [];
					if($scope.inFormMode() && text){
						eventAggregator.publish('creatingForm', JSON.parse(text));
					} else if(text || text === ""){
						setAnnotations(annotations);
						newItem = getItemAttributes(newItem);
						oldItem = getItemAttributes(oldItem);
						var items = {
							newItem: newItem,
							oldItem: oldItem
						};
						activeEditorSession(text, activeItem.cursorPosition, items);
					}
				}
				$scope.refreshSearch(newItem.path, oldItem.path);
			}

			$scope.$watch(watchActiveItem, Util.buildDebounceMethod(activeItemHandler, 200), true);

			Mousetrap.bind('mod+s', function(event) {
				event.preventDefault();
				$scope.DevToolsCtrl.saveComponentChange();
			});
		}
	};
});