var tree = {};

function getOnlyObjects(obj){
    return Object.keys(obj).filter(function(property){
        return property != "parent" && typeof obj[property] == "object";
    });
}

function clearTree(){
    tree = {};
}

var FileTree = {
    isFileAllowed: function(fileName){
        var filesToSave = [".js", ".html", ".css"];
        return filesToSave.some(function (type){
            return fileName.substring(fileName.length - type.length) == type;
        });
    },

    unsetParent: function(obj){
        var unsetParent = function(_obj){
            if(typeof _obj == "object"){
                delete _obj.parent;
                var properties = getOnlyObjects(_obj);
                properties.forEach(function(property){
                    _obj[property] = unsetParent(_obj[property]);
                });
            }
            return _obj;
        };
        return unsetParent(obj);
    },

    build : function(paths){
        var getUrl = function(path, name){
            return path.slice(0, path.indexOf(name) + name.length);
        };
        var buildNodeOfTree = function(parent, parentName, path, pathName, relativeUrl){
            if(!parent.hasOwnProperty(pathName)){
                var name = pathName;
                var nodeOfTree = {};
                if(FileTree.isFileAllowed(pathName)){
                    name = path.replace(/^.*[\\\/]/, '');
                    pathName = name;
                    nodeOfTree.relativeUrl = relativeUrl;
                    nodeOfTree.isFile = true;
                }
                nodeOfTree.name = nodeOfTree.label = name;
                nodeOfTree.isFolder = !nodeOfTree.isFile;
                nodeOfTree.showInDevTools = true;
                nodeOfTree.url = getUrl(path, name);
                nodeOfTree.parent = parentName;
                parent[pathName] = nodeOfTree;
            }
            return parent[pathName];
        };

        clearTree();
        paths.forEach(function(fileObj){
            var obj = tree;
            var path = fileObj.src;
            var arrayPath = path.split('/');

            arrayPath.forEach(function(pathName, index){
                obj = buildNodeOfTree(obj, arrayPath[index - 1], path, pathName, fileObj.relativeUrl);
            });
        });

        return tree;
    }
};